# Generated by Django 4.1.3 on 2022-12-17 18:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('inbox', '0002_alter_message_cc'),
    ]

    operations = [
        migrations.AddField(
            model_name='attachment',
            name='content_type',
            field=models.CharField(default=None, max_length=128, null=True),
        ),
    ]
