# Generated by Django 4.2.11 on 2024-08-13 04:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wiki', '0002_alter_pagemedia_entry'),
    ]

    operations = [
        migrations.AddField(
            model_name='page',
            name='perms',
            field=models.CharField(db_index=True, default=None, max_length=255, null=True),
        ),
    ]
