# Generated by Django 4.0.2 on 2022-02-22 01:26

from django.db import migrations, models
import django.db.models.deletion
import rest.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('account', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='PhonenumberInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('number', models.CharField(db_index=True, max_length=200)),
                ('owner_name', models.CharField(default=None, max_length=200, null=True)),
                ('owner_kind', models.CharField(default=None, max_length=200, null=True)),
                ('country', models.CharField(default=None, max_length=64, null=True)),
                ('carrier_name', models.CharField(default=None, max_length=64, null=True)),
                ('kind', models.CharField(default=None, max_length=64, null=True)),
                ('is_valid', models.BooleanField(default=False)),
                ('is_verified', models.BooleanField(default=False)),
                ('sms_allowed', models.BooleanField(default=False)),
            ],
            bases=(models.Model, rest.models.RestModel),
        ),
        migrations.CreateModel(
            name='SMS',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sid', models.CharField(blank=True, default=None, max_length=125, null=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('transport', models.CharField(blank=True, default=None, max_length=125, null=True)),
                ('endpoint', models.CharField(blank=True, db_index=True, default=None, max_length=200, null=True)),
                ('srcpoint', models.CharField(blank=True, default=None, max_length=200, null=True)),
                ('is_inbound', models.BooleanField(default=False)),
                ('status', models.CharField(db_index=True, default=None, max_length=64, null=True)),
                ('reason', models.TextField(default=None, null=True)),
                ('message', models.TextField()),
                ('by', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='telephony_sms_out', to='account.member')),
                ('group', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='telephony_sms', to='account.group')),
                ('to', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='telephony_sms_in', to='account.member')),
            ],
            bases=(models.Model, rest.models.RestModel),
        ),
    ]
