# Generated by Django 4.1.4 on 2023-11-27 22:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('telephony', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='sms',
            name='sid',
            field=models.CharField(blank=True, db_index=True, default=None, max_length=125, null=True),
        ),
    ]
