# Generated by Django 4.2.11 on 2024-05-13 02:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('incident', '0014_event_group_alter_rulecheck_index'),
    ]

    operations = [
        migrations.AddField(
            model_name='rule',
            name='title_template',
            field=models.CharField(default=None, max_length=200, null=True),
        ),
        migrations.AlterField(
            model_name='incident',
            name='state',
            field=models.IntegerField(choices=[(0, 'new'), (1, 'opened'), (2, 'paused'), (3, 'ignored'), (4, 'resolved'), (5, 'pending')], default=0),
        ),
    ]
