# Generated by Django 4.2.11 on 2024-05-13 02:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('inbox', '0004_mailtemplate'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mailbox',
            name='state',
            field=models.IntegerField(db_index=True, default=1),
        ),
    ]
