<!DOCTYPE html>
<html class="no-js" lang="en">
<head>
	<meta charset="utf-8">
	<title>{% block title %}Diginodes{% endblock %}</title>
	<meta name="viewport" content="width=device-width,initial-scale=1">
	
	<link rel="stylesheet" href="{{STATIC_URL}}css/base.css">
	<style type="text/css">
		{% block style %}
		{% endblock %}
		
		html,body,div,span,applet,object,iframe,h1,h2,h3,h4,h5,h6,p,blockquote,pre,a,abbr,acronym,address,big,cite,code,del,dfn,em,img,ins,kbd,q,s,samp,small,strike,strong,sub,sup,tt,var,b,u,i,center,dl,dt,dd,ol,ul,li,fieldset,form,label,legend,table,caption,tbody,tfoot,thead,tr,th,td,article,aside,canvas,details,embed,figure,figcaption,footer,header,hgroup,menu,nav,output,ruby,section,summary,time,mark,audio,video
			{
			margin: 0;
			padding: 0;
			border: 0;
			font-size: 100%;
			font: inherit;
			vertical-align: baseline;
		}
		/* HTML5 display-role reset for older browsers */
		article,aside,details,figcaption,figure,footer,header,hgroup,menu,nav,section
			{
			display: block;
		}

		ol,ul {
			list-style: none;
		}

		blockquote,q {
			quotes: none;
		}

		blockquote:before,blockquote:after,q:before,q:after {
			content: '';
			content: none;
		}

		table {
			border-collapse: collapse;
			border-spacing: 0;
			min-width: 800px;
		}
		
		#header-bar {
			
			height: 32px;
			width: 100%;
			color: #C7C7C7;
			background : #1C1C1C;
			box-shadow: 0 4px 4px -2px #000000;
			-moz-box-shadow: 0 4px 4px -2px #000000;
			-webkit-box-shadow: 0 4px 4px -2px #000000;
		}
		
		.logo-labels {
			float : left;
			position : relative;
			overflow : hidden;
			width: 100%;
			min-width: 800px;
			height : 32px;
		}

		.logo-labels h1 {
			color: #62A3FA;
			position : absolute;
			left : 4px;
			top : 4px;
			font-size : 18px;
			width : 165px;
			height : 20px;
		}

		.logo-labels h1 a, .logo-labels h1 a:link, .logo-labels h1 a:visited, .logo-labels h1 a:active {
			display : block;
			width : 100%;
			height : 100%;
			cursor: pointer;
		}

		.logo-labels ul {
			float : right;
			margin : 0;
			margin-right: 8px;
			padding : 6px 0 0;
			list-style : none;
			width : auto;
		}

		.logo-labels ul li {
			float : left;
			background : none;
			padding : 0 0 0 8px;
			font : 11px Arial;
			line-height : 1.9em;
			cursor : pointer;
			color : #FFFFFF;
			height : 22px;
		}

		.logo-labels ul li a {
			color: white;
			text-decoration: none;
		}

		.logo-labels ul li select {
			width: 200px;
		}

		.footer {
			position: fixed;
			bottom: 0;
			left: 0;
			height: 24px;
			width: 100%;
			background: #1C1C1C;
			font-size: 10px;
			line-height: 12px;
			padding-left: 4px;
		}

		.footer a, .footer a:link, .footer a:active, .footer a:visited {
			color : #D2D2D2;
			text-decoration : none;
		}

		.footer a:hover {
			color : #949ea8;
			text-decoration : none;
		}

		.footer .split{
			color: white;
			text-align: center;
			width : 50%;
		}
		
		html, body {
			height: 100%;
			background: #ffffff;
			font-family: Arial;
		}
		.content {
			height: 100%;
			width: 100%;
			padding: 12px;
		}

	</style>

	<script src="{{STATIC_URL}}lib/jquery-1.6.1.min.js"></script>
	<script src="{{STATIC_URL}}js/all.js"></script>
	<script src="{{STATIC_URL}}js/jquery.twin.js"></script>
	
	{% block head %}
	{% endblock %}
	
</head>
<body>
	<div id="header-bar">
		<div class="logo-labels">
			<h1>diginodes</h1>
		</div>
	</div>

	<div class="content">
		{% block content %}
		{% endblock %}	
		{% block main %}
		{% endblock %}	
	</div>

	<div class="footer">
		{% block footer %}
		<div class="split">&#169; Copyright <a href="#">twintechs.com</a></div> 
		<div class="split right">

		</div> 
		{% endblock %}
	</div>
</body>
</html>
