# Generated by Django 4.1.4 on 2023-11-28 23:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('account', '0016_authsession_buid'),
    ]

    operations = [
        migrations.RenameField(
            model_name='member',
            old_name='requires_topt',
            new_name='requires_totp',
        ),
    ]
