# Generated by Django 4.1.4 on 2023-01-06 22:15

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('incident', '0005_incident_component_alter_incident_state'),
    ]

    operations = [
        migrations.DeleteModel(
            name='EventMetaData',
        ),
    ]
