# Generated by Django 4.1.4 on 2024-04-15 13:45

from django.db import migrations, models
import django.db.models.deletion
import rest.fields
import rest.models.base
import rest.models.metadata


class Migration(migrations.Migration):

    dependencies = [
        ('account', '0019_group_location'),
    ]

    operations = [
        migrations.CreateModel(
            name='CloudCredentials',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('modified', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(blank=True, default=None, max_length=128, null=True)),
                ('uuid', models.CharField(blank=True, db_index=True, default=None, max_length=64, null=True)),
                ('state', models.IntegerField(db_index=True, default=1)),
                ('credentials', rest.fields.JSONField(default=None, null=True)),
                ('group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='cloud_credentials', to='account.group')),
            ],
            bases=(models.Model, rest.models.base.RestModel, rest.models.metadata.MetaDataModel),
        ),
        migrations.CreateModel(
            name='CloudCredentialsMetaData',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('category', models.CharField(blank=True, db_index=True, default=None, max_length=32, null=True)),
                ('key', models.CharField(db_index=True, max_length=80)),
                ('value_format', models.CharField(max_length=16)),
                ('value', models.TextField()),
                ('int_value', models.IntegerField(blank=True, default=None, null=True)),
                ('float_value', models.IntegerField(blank=True, default=None, null=True)),
                ('parent', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='properties', to='account.cloudcredentials')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
