# Generated by Django 4.1.4 on 2023-11-07 12:44

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('account', '0013_memberdevice_ip'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notificationmemberrecord',
            name='member',
            field=models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notifications', to='account.member'),
        ),
    ]
