# Generated by Django 4.1.3 on 2022-12-15 05:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('inbox', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='message',
            name='cc',
            field=models.TextField(default=None, null=True),
        ),
    ]
